@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013, 2014, 2015, 2016 Laboratoire de Recherche et
-// Développement de l'Epita (LRDE).
+// Copyright (C) 2013-2018 Laboratoire de Recherche et Développement
+// de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -40,31 +40,35 @@
 #include "common_conv.hh"
 #include "common_hoaread.hh"
 
-#include <spot/twaalgos/product.hh>
-#include <spot/twaalgos/isdet.hh>
-#include <spot/twaalgos/stutter.hh>
-#include <spot/twaalgos/isunamb.hh>
 #include <spot/misc/optionmap.hh>
-#include <spot/misc/timer.hh>
 #include <spot/misc/random.hh>
+#include <spot/misc/timer.hh>
 #include <spot/parseaut/public.hh>
 #include <spot/tl/exclusive.hh>
-#include <spot/twaalgos/remprop.hh>
-#include <spot/twaalgos/randomize.hh>
 #include <spot/twaalgos/are_isomorphic.hh>
 #include <spot/twaalgos/canonicalize.hh>
-#include <spot/twaalgos/mask.hh>
-#include <spot/twaalgos/sepsets.hh>
-#include <spot/twaalgos/stripacc.hh>
-#include <spot/twaalgos/remfin.hh>
+#include <spot/twaalgos/cobuchi.hh>
 #include <spot/twaalgos/cleanacc.hh>
 #include <spot/twaalgos/dtwasat.hh>
-#include <spot/twaalgos/complement.hh>
-#include <spot/twaalgos/strength.hh>
-#include <spot/twaalgos/hoa.hh>
-#include <spot/twaalgos/sccinfo.hh>
-#include <spot/twaalgos/isweakscc.hh>
+#include <spot/twaalgos/dualize.hh>
 #include <spot/twaalgos/gtec/gtec.hh>
+#include <spot/twaalgos/hoa.hh>
+#include <spot/twaalgos/isdet.hh>
+#include <spot/twaalgos/isunamb.hh>
+#include <spot/twaalgos/isweakscc.hh>
+#include <spot/twaalgos/langmap.hh>
+#include <spot/twaalgos/mask.hh>
+#include <spot/twaalgos/product.hh>
+#include <spot/twaalgos/randomize.hh>
+#include <spot/twaalgos/remfin.hh>
+#include <spot/twaalgos/remprop.hh>
+#include <spot/twaalgos/sccinfo.hh>
+#include <spot/twaalgos/sepsets.hh>
+#include <spot/twaalgos/split.hh>
+#include <spot/twaalgos/strength.hh>
+#include <spot/twaalgos/stripacc.hh>
+#include <spot/twaalgos/stutter.hh>
+#include <spot/twaalgos/sum.hh>
 #include <spot/twaalgos/totgba.hh>
 
 static const char argp_program_doc[] ="\
@@ -79,6 +83,7 @@ enum {
   OPT_ACC_SCCS = 256,
   OPT_ACC_SETS,
   OPT_ACCEPT_WORD,
+  OPT_ACCEPTANCE_IS,
   OPT_AP_N,
   OPT_ARE_ISOMORPHIC,
   OPT_CLEAN_ACC,
@@ -86,8 +91,10 @@ enum {
   OPT_COMPLEMENT,
   OPT_COMPLEMENT_ACC,
   OPT_COUNT,
-  OPT_DECOMPOSE_STRENGTH,
+  OPT_DCA,
+  OPT_DECOMPOSE_SCC,
   OPT_DESTUT,
+  OPT_DUALIZE,
   OPT_DNF_ACC,
   OPT_EDGES,
   OPT_EQUIVALENT_TO,
@@ -98,18 +105,22 @@ enum {
   OPT_HIGHLIGHT_NONDET_EDGES,
   OPT_HIGHLIGHT_NONDET_STATES,
   OPT_HIGHLIGHT_WORD,
+  OPT_HIGHLIGHT_LANGUAGES,
   OPT_INSTUT,
   OPT_INCLUDED_IN,
   OPT_INHERENTLY_WEAK_SCCS,
   OPT_INTERSECT,
+  OPT_IS_ALTERNATING,
   OPT_IS_COMPLETE,
   OPT_IS_DETERMINISTIC,
   OPT_IS_EMPTY,
+  OPT_IS_INHERENTLY_WEAK,
+  OPT_IS_SEMI_DETERMINISTIC,
   OPT_IS_STUTTER_INVARIANT,
   OPT_IS_TERMINAL,
   OPT_IS_UNAMBIGUOUS,
+  OPT_IS_VERY_WEAK,
   OPT_IS_WEAK,
-  OPT_IS_INHERENTLY_WEAK,
   OPT_KEEP_STATES,
   OPT_MASK_ACC,
   OPT_MERGE,
@@ -128,9 +139,14 @@ enum {
   OPT_SCCS,
   OPT_SEED,
   OPT_SEP_SETS,
+  OPT_SIMPL_ACC,
   OPT_SIMPLIFY_EXCLUSIVE_AP,
+  OPT_SPLIT_EDGES,
   OPT_STATES,
+  OPT_STREETT_LIKE,
   OPT_STRIPACC,
+  OPT_SUM_OR,
+  OPT_SUM_AND,
   OPT_TERMINAL_SCCS,
   OPT_TRIV_SCCS,
   OPT_USED_AP_N,
@@ -138,6 +154,8 @@ enum {
   OPT_WEAK_SCCS,
 };
 
+#define DOC(NAME, TXT) NAME, 0, nullptr, OPTION_DOC | OPTION_NO_USAGE, TXT, 0
+
 static const argp_option options[] =
   {
     /**************************************************/
@@ -157,6 +175,8 @@ static const argp_option options[] =
     { "unused-ap", OPT_UNUSED_AP_N, "RANGE", 0,
       "match automata with a number of declared, but unused atomic "
       "propositions in RANGE", 0 },
+    { "acceptance-is", OPT_ACCEPTANCE_IS, "NAME|FORMULA", 0,
+      "match automata with given accetance condition", 0 },
     { "are-isomorphic", OPT_ARE_ISOMORPHIC, "FILENAME", 0,
       "keep automata that are isomorphic to the automaton in FILENAME", 0 },
     { "isomorphic", 0, nullptr, OPTION_ALIAS | OPTION_HIDDEN, nullptr, 0 },
@@ -167,6 +187,8 @@ static const argp_option options[] =
       "keep complete automata", 0 },
     { "is-deterministic", OPT_IS_DETERMINISTIC, nullptr, 0,
       "keep deterministic automata", 0 },
+    { "is-semi-deterministic", OPT_IS_SEMI_DETERMINISTIC, nullptr, 0,
+      "keep semi-deterministic automata", 0 },
     { "is-empty", OPT_IS_EMPTY, nullptr, 0,
       "keep automata with an empty language", 0 },
     { "is-stutter-invariant", OPT_IS_STUTTER_INVARIANT, nullptr, 0,
@@ -179,6 +201,10 @@ static const argp_option options[] =
       "keep only weak automata", 0 },
     { "is-inherently-weak", OPT_IS_INHERENTLY_WEAK, nullptr, 0,
       "keep only inherently weak automata", 0 },
+    { "is-very-weak", OPT_IS_VERY_WEAK, nullptr, 0,
+      "keep only very-weak automata", 0 },
+    { "is-alternating", OPT_IS_ALTERNATING, nullptr, 0,
+      "keep only automata using universal branching", 0 },
     { "intersect", OPT_INTERSECT, "FILENAME", 0,
       "keep automata whose languages have an non-empty intersection with"
       " the automaton from FILENAME", 0 },
@@ -253,6 +279,9 @@ static const argp_option options[] =
       "initial state.  Implies --remove-unreachable-states.", 0 },
     { "dnf-acceptance", OPT_DNF_ACC, nullptr, 0,
       "put the acceptance condition in Disjunctive Normal Form", 0 },
+    { "streett-like", OPT_STREETT_LIKE, nullptr, 0,
+      "convert to an automaton with Streett-like acceptance. Works only with "
+      "acceptance condition in DNF", 0 },
     { "cnf-acceptance", OPT_CNF_ACC, nullptr, 0,
       "put the acceptance condition in Conjunctive Normal Form", 0 },
     { "remove-fin", OPT_REM_FIN, nullptr, 0,
@@ -279,9 +308,14 @@ static const argp_option options[] =
     { "complement-acceptance", OPT_COMPLEMENT_ACC, nullptr, 0,
       "complement the acceptance condition (without touching the automaton)",
       0 },
-    { "decompose-strength", OPT_DECOMPOSE_STRENGTH, "t|w|s", 0,
+    { "decompose-scc", OPT_DECOMPOSE_SCC, "t|w|s|N|aN", 0,
       "extract the (t) terminal, (w) weak, or (s) strong part of an automaton"
-      " (letters may be combined to combine more strengths in the output)", 0 },
+      " or (N) the subautomaton leading to the Nth SCC, or (aN) to the Nth "
+      "accepting SCC (option can be combined with commas to extract multiple "
+      "parts)", 0 },
+    { "decompose-strength", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
+    { "dualize", OPT_DUALIZE, nullptr, 0,
+      "dualize each automaton", 0 },
     { "exclusive-ap", OPT_EXCLUSIVE_AP, "AP,AP,...", 0,
       "if any of those APs occur in the automaton, restrict all edges to "
       "ensure two of them may not be true at the same time.  Use this option "
@@ -301,14 +335,31 @@ static const argp_option options[] =
     { "remove-dead-states", OPT_REM_DEAD, nullptr, 0,
       "remove states that are unreachable, or that cannot belong to an "
       "infinite path", 0 },
+    { "simplify-acceptance", OPT_SIMPL_ACC, nullptr, 0,
+      "simplify the acceptance condition by merging identical acceptance sets "
+      "and by simplifying some terms containing complementary sets", 0 },
+    { "split-edges", OPT_SPLIT_EDGES, nullptr, 0,
+      "split edges into transitions labeled by conjunctions of all atomic "
+      "propositions, so they can be read as letters", 0 },
+    { "sum", OPT_SUM_OR, "FILENAME", 0,
+      "build the sum with the automaton in FILENAME "
+      "to sum languages", 0 },
+    { "sum-or", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
+    { "sum-and", OPT_SUM_AND, "FILENAME", 0,
+      "build the sum with the automaton in FILENAME "
+      "to intersect languages", 0 },
     { "separate-sets", OPT_SEP_SETS, nullptr, 0,
       "if both Inf(x) and Fin(x) appear in the acceptance condition, replace "
       "Fin(x) by a new Fin(y) and adjust the automaton", 0 },
     { "sat-minimize", OPT_SAT_MINIMIZE, "options", OPTION_ARG_OPTIONAL,
       "minimize the automaton using a SAT solver (only works for deterministic"
-      " automata)", 0 },
-    /**************************************************/
-    { nullptr, 0, nullptr, 0, "Decorations (for -d and -H1.1 output):", 8 },
+      " automata). Supported options are acc=STRING, states=N, max-states=N, "
+      "sat-incr=N, sat-incr-steps=N, sat-langmap, sat-naive, colored, preproc=N"
+      ". Spot uses by default its PicoSAT distribution but an external SAT"
+      "solver can be set thanks to the SPOT_SATSOLVER environment variable"
+      "(see spot-x)."
+      , 0 },
+    { nullptr, 0, nullptr, 0, "Decorations (for -d and -H1.1 output):", 9 },
     { "highlight-nondet-states", OPT_HIGHLIGHT_NONDET_STATES, "NUM",
       OPTION_ARG_OPTIONAL, "highlight nondeterministic states with color NUM",
       0 },
@@ -320,6 +371,8 @@ static const argp_option options[] =
       "highlight nondeterministic states and edges with color NUM", 0},
     { "highlight-word", OPT_HIGHLIGHT_WORD, "[NUM,]WORD", 0,
       "highlight one run matching WORD using color NUM", 0},
+    { "highlight-languages", OPT_HIGHLIGHT_LANGUAGES, nullptr, 0 ,
+      "highlight states that recognize identical languages", 0},
     /**************************************************/
     { nullptr, 0, nullptr, 0,
       "If any option among --small, --deterministic, or --any is given, "
@@ -369,6 +422,99 @@ static gra_type const gra_types[] =
 };
 ARGMATCH_VERIFY(gra_args, gra_types);
 
+enum acc_type {
+  ACC_Any = 0,
+  ACC_Given,
+  ACC_tt,
+  ACC_ff,
+  ACC_Buchi,
+  ACC_GenBuchi,
+  ACC_CoBuchi,
+  ACC_GenCoBuchi,
+  ACC_Rabin,
+  ACC_Streett,
+  ACC_GenRabin,
+  ACC_GenStreett,
+  ACC_RabinLike,
+  ACC_StreettLike,
+  ACC_Parity,
+  ACC_ParityMin,
+  ACC_ParityMax,
+  ACC_ParityOdd,
+  ACC_ParityEven,
+  ACC_ParityMinOdd,
+  ACC_ParityMaxOdd,
+  ACC_ParityMinEven,
+  ACC_ParityMaxEven,
+  ACC_FinLess,
+};
+static acc_type opt_acceptance_is = ACC_Any;
+spot::acc_cond opt_acceptance_is_given;
+static char const *const acc_is_args[] =
+  {
+    "any",
+    "t", "all",
+    "f", "none",
+    "Buchi", "Büchi",
+    "generalized-Buchi", "generalized-Büchi",
+    "genBuchi", "genBüchi", "gen-Buchi", "gen-Büchi",
+    "coBuchi", "coBüchi", "co-Buchi", "co-Büchi",
+    "generalized-coBuchi", "generalized-coBüchi",
+    "generalized-co-Buchi", "generalized-co-Büchi",
+    "gen-coBuchi", "gen-coBüchi",
+    "gen-co-Buchi", "gen-co-Büchi",
+    "gencoBuchi", "gencoBüchi",
+    "Rabin",
+    "Streett",
+    "generalized-Rabin", "gen-Rabin", "genRabin",
+    "generalized-Streett", "gen-Streett", "genStreett",
+    "Streett-like",
+    "Rabin-like",
+    "parity",
+    "parity min", "parity-min",
+    "parity max", "parity-max",
+    "parity odd", "parity-odd",
+    "parity even", "parity-even",
+    "parity min even", "parity-min-even",
+    "parity min odd", "parity-min-odd",
+    "parity max even", "parity-max-even",
+    "parity max odd", "parity-max-odd",
+    "Fin-less", "Finless",
+    nullptr,
+  };
+static acc_type const acc_is_types[] =
+  {
+    ACC_Any,
+    ACC_tt, ACC_tt,
+    ACC_ff, ACC_ff,
+    ACC_Buchi, ACC_Buchi,
+    ACC_GenBuchi, ACC_GenBuchi,
+    ACC_GenBuchi, ACC_GenBuchi, ACC_GenBuchi, ACC_GenBuchi,
+    ACC_CoBuchi, ACC_CoBuchi, ACC_CoBuchi, ACC_CoBuchi,
+    ACC_GenCoBuchi, ACC_GenCoBuchi,
+    ACC_GenCoBuchi, ACC_GenCoBuchi,
+    ACC_GenCoBuchi, ACC_GenCoBuchi,
+    ACC_GenCoBuchi, ACC_GenCoBuchi,
+    ACC_GenCoBuchi, ACC_GenCoBuchi,
+    ACC_Rabin,
+    ACC_Streett,
+    ACC_GenRabin, ACC_GenRabin, ACC_GenRabin,
+    ACC_GenStreett, ACC_GenStreett, ACC_GenStreett,
+    ACC_StreettLike,
+    ACC_RabinLike,
+    ACC_Parity,
+    ACC_ParityMin, ACC_ParityMin,
+    ACC_ParityMax, ACC_ParityMax,
+    ACC_ParityOdd, ACC_ParityOdd,
+    ACC_ParityEven, ACC_ParityEven,
+    ACC_ParityMinEven, ACC_ParityMinEven,
+    ACC_ParityMinOdd, ACC_ParityMinOdd,
+    ACC_ParityMaxEven, ACC_ParityMaxEven,
+    ACC_ParityMaxOdd, ACC_ParityMaxOdd,
+    ACC_FinLess, ACC_FinLess,
+  };
+ARGMATCH_VERIFY(acc_is_args, acc_is_types);
+
 enum gsa_type { GSA_NO = 0, GSA_SHARE_FIN = 1, GSA_UNIQUE_FIN = 2 };
 static gsa_type opt_gsa = GSA_NO;
 static char const *const gsa_args[] =
@@ -390,6 +536,8 @@ static struct opt_t
   spot::bdd_dict_ptr dict = spot::make_bdd_dict();
   spot::twa_graph_ptr product_and = nullptr;
   spot::twa_graph_ptr product_or = nullptr;
+  spot::twa_graph_ptr sum_and = nullptr;
+  spot::twa_graph_ptr sum_or = nullptr;
   spot::twa_graph_ptr intersect = nullptr;
   spot::twa_graph_ptr included_in = nullptr;
   spot::twa_graph_ptr equivalent_pos = nullptr;
@@ -406,12 +554,15 @@ static struct opt_t
 }* opt;
 
 static bool opt_merge = false;
+static bool opt_is_alternating = false;
 static bool opt_is_complete = false;
 static bool opt_is_deterministic = false;
+static bool opt_is_semi_deterministic = false;
 static bool opt_is_unambiguous = false;
 static bool opt_is_terminal = false;
 static bool opt_is_weak = false;
 static bool opt_is_inherently_weak = false;
+static bool opt_is_very_weak = false;
 static bool opt_is_stutter_invariant = false;
 static bool opt_invert = false;
 static range opt_states = { 0, std::numeric_limits<int>::max() };
@@ -446,23 +597,29 @@ static bool opt_rem_fin = false;
 static bool opt_clean_acc = false;
 static bool opt_complement = false;
 static bool opt_complement_acc = false;
-static char* opt_decompose_strength = nullptr;
+static char* opt_decompose_scc = nullptr;
+static bool opt_dualize = false;
 static spot::acc_cond::mark_t opt_mask_acc = 0U;
 static std::vector<bool> opt_keep_states = {};
 static unsigned int opt_keep_states_initial = 0;
+static bool opt_simpl_acc = false;
 static bool opt_simplify_exclusive_ap = false;
 static bool opt_rem_dead = false;
 static bool opt_rem_unreach = false;
 static bool opt_rem_unused_ap = false;
 static bool opt_sep_sets = false;
+static bool opt_split_edges = false;
 static const char* opt_sat_minimize = nullptr;
 static int opt_highlight_nondet_states = -1;
 static int opt_highlight_nondet_edges = -1;
+static bool opt_highlight_languages = false;
+static bool opt_dca = false;
+static bool opt_streett_like = false;
 
 static spot::twa_graph_ptr
-ensure_deterministic(const spot::twa_graph_ptr& aut)
+ensure_deterministic(const spot::twa_graph_ptr& aut, bool nonalt = false)
 {
-  if (spot::is_deterministic(aut))
+  if ((!nonalt || aut->is_existential()) && spot::is_universal(aut))
     return aut;
   spot::postprocessor p;
   p.set_type(spot::postprocessor::Generic);
@@ -522,6 +679,45 @@ parse_opt(int key, char* arg, struct argp_state*)
                 e.what());
         }
       break;
+    case OPT_ACCEPTANCE_IS:
+      {
+        auto res = argmatch(arg, acc_is_args,
+                            (char const*) acc_is_types, sizeof(acc_type));
+        if (res >= 0)
+          opt_acceptance_is = acc_is_types[res];
+        else
+          {
+            try
+              {
+                opt_acceptance_is_given =
+                  spot::acc_cond(spot::acc_cond::acc_code(arg));
+                opt_acceptance_is = ACC_Given;
+              }
+            catch (const spot::parse_error& err)
+              {
+                std::cerr << program_name << ": failed to parse '" << arg
+                          << ("' as an acceptance formula for "
+                              "--acceptance-is\n\t")
+                          << err.what()
+                          << ("\nIf you do not want to supply a formula, the "
+                              "following names are recognized:");
+                acc_type last_val = ACC_Any;
+                for (int i = 0; acc_is_args[i]; i++)
+                  if ((i == 0) || last_val != acc_is_types[i])
+                    {
+                      std::cerr << "\n  - '" << acc_is_args[i] << '\'';
+                      last_val = acc_is_types[i];
+                    }
+                  else
+                    {
+                      std::cerr << ", '" << acc_is_args[i] << '"';
+                    }
+                std::cerr << std::flush;
+                exit(2);
+              }
+          }
+      }
+      break;
     case OPT_ARE_ISOMORPHIC:
       opt->are_isomorphic = read_automaton(arg, opt->dict);
       break;
@@ -533,21 +729,36 @@ parse_opt(int key, char* arg, struct argp_state*)
       opt_cnf_acc = true;
       break;
     case OPT_COMPLEMENT:
+      if (opt_dualize)
+        error(2, 0, "either --complement or --dualize options"
+                    " can be given, not both");
       opt_complement = true;
       break;
     case OPT_COMPLEMENT_ACC:
       opt_complement_acc = true;
       break;
-    case OPT_DECOMPOSE_STRENGTH:
-      opt_decompose_strength = arg;
+     case OPT_DCA:
+      opt_dca = true;
+      break;
+    case OPT_DECOMPOSE_SCC:
+      opt_decompose_scc = arg;
       break;
     case OPT_DESTUT:
       opt_destut = true;
       break;
+    case OPT_DUALIZE:
+      if (opt_complement)
+        error(2, 0, "either --complement or --dualize options"
+                    " can be given, not both");
+      opt_dualize = true;
+      break;
     case OPT_DNF_ACC:
       opt_dnf_acc = true;
       opt_cnf_acc = false;
       break;
+    case OPT_STREETT_LIKE:
+      opt_streett_like = true;
+      break;
     case OPT_EDGES:
       opt_edges = parse_range(arg, 0, std::numeric_limits<int>::max());
       break;
@@ -559,7 +770,7 @@ parse_opt(int key, char* arg, struct argp_state*)
         error(2, 0, "only one --equivalent-to option can be given");
       opt->equivalent_pos = read_automaton(arg, opt->dict);
       opt->equivalent_neg =
-        spot::dtwa_complement(ensure_deterministic(opt->equivalent_pos));
+        spot::dualize(ensure_deterministic(opt->equivalent_pos, true));
       break;
     case OPT_GENERALIZED_RABIN:
       if (arg)
@@ -619,7 +830,9 @@ parse_opt(int key, char* arg, struct argp_state*)
           }
       }
       break;
-
+    case OPT_HIGHLIGHT_LANGUAGES:
+      opt_highlight_languages = true;
+      break;
     case OPT_INSTUT:
       if (!arg || (arg[0] == '1' && arg[1] == 0))
         opt_instut = 1;
@@ -630,8 +843,8 @@ parse_opt(int key, char* arg, struct argp_state*)
       break;
     case OPT_INCLUDED_IN:
       {
-        auto aut = ensure_deterministic(read_automaton(arg, opt->dict));
-        aut = spot::dtwa_complement(aut);
+        auto aut = ensure_deterministic(read_automaton(arg, opt->dict), true);
+        aut = spot::dualize(aut);
         if (!opt->included_in)
           opt->included_in = aut;
         else
@@ -646,6 +859,9 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_INTERSECT:
       opt->intersect = read_automaton(arg, opt->dict);
       break;
+    case OPT_IS_ALTERNATING:
+      opt_is_alternating = true;
+      break;
     case OPT_IS_COMPLETE:
       opt_is_complete = true;
       break;
@@ -658,6 +874,12 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_IS_INHERENTLY_WEAK:
       opt_is_inherently_weak = true;
       break;
+    case OPT_IS_VERY_WEAK:
+      opt_is_very_weak = true;
+      break;
+    case OPT_IS_SEMI_DETERMINISTIC:
+      opt_is_semi_deterministic = true;
+      break;
     case OPT_IS_STUTTER_INVARIANT:
       opt_is_stutter_invariant = true;
       break;
@@ -685,7 +907,6 @@ parse_opt(int key, char* arg, struct argp_state*)
               opt_keep_states.resize(res + 1, false);
             opt_keep_states[res] = true;
           }
-        opt_rem_unreach = true;
         break;
       }
     case OPT_MERGE:
@@ -796,16 +1017,42 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_SEP_SETS:
       opt_sep_sets = true;
       break;
+    case OPT_SIMPL_ACC:
+      opt_simpl_acc = true;
+      break;
     case OPT_SIMPLIFY_EXCLUSIVE_AP:
       opt_simplify_exclusive_ap = true;
       opt_merge = true;
       break;
+    case OPT_SPLIT_EDGES:
+      opt_split_edges = true;
+      break;
     case OPT_STATES:
       opt_states = parse_range(arg, 0, std::numeric_limits<int>::max());
       break;
     case OPT_STRIPACC:
       opt_stripacc = true;
       break;
+    case OPT_SUM_OR:
+      {
+        auto a = read_automaton(arg, opt->dict);
+        if (!opt->sum_or)
+          opt->sum_or = std::move(a);
+        else
+          opt->sum_or = spot::sum(std::move(opt->sum_or),
+                                  std::move(a));
+      }
+      break;
+    case OPT_SUM_AND:
+      {
+        auto a = read_automaton(arg, opt->dict);
+        if (!opt->sum_and)
+          opt->sum_and = std::move(a);
+        else
+          opt->sum_and = spot::sum_and(std::move(opt->sum_and),
+                                       std::move(a));
+      }
+      break;
     case OPT_TERMINAL_SCCS:
       opt_terminal_sccs = parse_range(arg, 0, std::numeric_limits<int>::max());
       opt_terminal_sccs_set = true;
@@ -858,114 +1105,114 @@ static int unused_ap(const spot::const_twa_graph_ptr& aut)
 
 namespace
 {
-  class hoa_processor final: public job_processor
+  static
+  bool match_acceptance(spot::twa_graph_ptr aut)
+  {
+    auto& acc = aut->acc();
+    switch (opt_acceptance_is)
+      {
+      case ACC_Any:
+        return true;
+      case ACC_Given:
+        return acc == opt_acceptance_is_given;
+      case ACC_tt:
+        return acc.is_t();
+      case ACC_ff:
+        return acc.is_f();
+      case ACC_Buchi:
+        return acc.is_buchi();
+      case ACC_GenBuchi:
+        return acc.is_generalized_buchi();
+      case ACC_CoBuchi:
+        return acc.is_co_buchi();
+      case ACC_GenCoBuchi:
+        return acc.is_generalized_co_buchi();
+      case ACC_Rabin:
+        return acc.is_rabin() >= 0;
+      case ACC_Streett:
+        return acc.is_streett() >= 0;
+      case ACC_GenRabin:
+        {
+          std::vector<unsigned> p;
+          return acc.is_generalized_rabin(p);
+        }
+      case ACC_GenStreett:
+        {
+          std::vector<unsigned> p;
+          return acc.is_generalized_streett(p);
+        }
+      case ACC_RabinLike:
+        {
+          std::vector<spot::acc_cond::rs_pair> p;
+          return acc.is_rabin_like(p);
+        }
+      case ACC_StreettLike:
+        {
+          std::vector<spot::acc_cond::rs_pair> p;
+          return acc.is_streett_like(p);
+        }
+      case ACC_Parity:
+      case ACC_ParityMin:
+      case ACC_ParityMax:
+      case ACC_ParityOdd:
+      case ACC_ParityEven:
+      case ACC_ParityMinOdd:
+      case ACC_ParityMaxOdd:
+      case ACC_ParityMinEven:
+      case ACC_ParityMaxEven:
+        {
+          bool max;
+          bool odd;
+          bool is_p = acc.is_parity(max, odd, true);
+          if (!is_p)
+            return false;
+          switch (opt_acceptance_is)
+            {
+            case ACC_Parity:
+              return true;
+            case ACC_ParityMin:
+              return !max;
+            case ACC_ParityMax:
+              return max;
+            case ACC_ParityOdd:
+              return odd;
+            case ACC_ParityEven:
+              return !odd;
+            case ACC_ParityMinOdd:
+              return !max && odd;
+            case ACC_ParityMaxOdd:
+              return max && odd;
+            case ACC_ParityMinEven:
+              return !max && !odd;
+            case ACC_ParityMaxEven:
+              return max && !odd;
+            default:
+              SPOT_UNREACHABLE();
+            }
+        }
+      case ACC_FinLess:
+        return !acc.uses_fin_acceptance();
+      }
+    SPOT_UNREACHABLE();
+  }
+
+
+  struct autfilt_processor: hoa_processor
   {
   private:
     spot::postprocessor& post;
     automaton_printer printer;
   public:
-
-    hoa_processor(spot::postprocessor& post)
-      : post(post), printer(aut_input)
+    autfilt_processor(spot::postprocessor& post, spot::bdd_dict_ptr dict)
+      : hoa_processor(dict), post(post), printer(aut_input)
     {
     }
 
     int
-    process_formula(spot::formula, const char*, int) override
+    process_automaton(const spot::const_parsed_aut_ptr& haut) override
     {
-      SPOT_UNREACHABLE();
-    }
-
-    int process_string(const std::string& input, const char* filename,
-                       int linenum) override
-    {
-      std::ostringstream loc;
-      loc << filename << ':' << linenum;
-      std::string locstr = loc.str();
-      return process_automaton_stream
-        (spot::automaton_stream_parser(input.c_str(), locstr, opt_parse),
-         locstr.c_str());
-    }
-
-    int
-    aborted(const spot::const_parsed_aut_ptr& h, const char* filename)
-    {
-      std::cerr << filename << ':' << h->loc << ": aborted input automaton\n";
-      return 2;
-    }
-
-    int
-    process_file(const char* filename) override
-    {
-      // If we have a filename like "foo/NN" such
-      // that:
-      // ① foo/NN is not a file,
-      // ② NN is a number,
-      // ③ foo is a file,
-      // then it means we want to open foo as
-      // a CSV file and process column NN.
-      if (const char* slash = strrchr(filename, '/'))
-        {
-          char* end;
-          errno = 0;
-          long int col = strtol(slash + 1, &end, 10);
-          if (errno == 0 && !*end && col != 0)
-            {
-              struct stat buf;
-              if (stat(filename, &buf) != 0)
-                {
-                  col_to_read = col;
-                  if (real_filename)
-                    free(real_filename);
-                  real_filename = strndup(filename, slash - filename);
-
-                  // Special case for stdin.
-                  if (real_filename[0] == '-' && real_filename[1] == 0)
-                    return process_stream(std::cin, real_filename);
-
-                  std::ifstream input(real_filename);
-                  if (input)
-                    return process_stream(input, real_filename);
-
-                  error(2, errno, "cannot open '%s' nor '%s'",
-                        filename, real_filename);
-                }
-            }
-        }
-
-      return process_automaton_stream(spot::automaton_stream_parser(filename,
-                                                                    opt_parse),
-                                      filename);
-    }
-
-    int process_automaton_stream(spot::automaton_stream_parser&& hp,
-                                 const char* filename)
-    {
-      int err = 0;
-      while (!abort_run)
-        {
-          auto haut = hp.parse(opt->dict);
-          if (!haut->aut && haut->errors.empty())
-            break;
-          if (haut->format_errors(std::cerr))
-            err = 2;
-          if (!haut->aut)
-            error(2, 0, "failed to read automaton from %s", filename);
-          else if (haut->aborted)
-            err = std::max(err, aborted(haut, filename));
-          else
-            process_automaton(haut, filename);
-        }
-      return err;
-    }
-
-
-    int
-    process_automaton(const spot::const_parsed_aut_ptr& haut,
-                      const char* filename)
-    {
-      spot::stopwatch sw;
-      sw.start();
+      spot::process_timer timer;
+      timer.start();
 
       // If --stats or --name is used, duplicate the automaton so we
       // never modify the original automaton (e.g. with
@@ -980,8 +1227,12 @@ namespace
         spot::strip_acceptance_here(aut);
       if (opt_merge)
         aut->merge_edges();
-      if (opt_clean_acc || opt_rem_fin)
+
+      if (opt_simpl_acc)
+        simplify_acceptance_here(aut);
+      else if (opt_clean_acc)
         cleanup_acceptance_here(aut);
+
       if (opt_sep_sets)
         separate_sets_here(aut);
       if (opt_complement_acc)
@@ -1010,8 +1261,14 @@ namespace
           matched &= opt_unused_ap_n.contains(unused);
           matched &= opt_used_ap_n.contains(aut->ap().size() - unused);
         }
+      if (matched && opt_is_alternating)
+        matched &= !aut->is_existential();
       if (matched && opt_is_complete)
         matched &= is_complete(aut);
+
+      if (matched && opt_acceptance_is)
+        matched = match_acceptance(aut);
+
       if (matched && (opt_sccs_set | opt_art_sccs_set))
         {
           spot::scc_info si(aut);
@@ -1057,11 +1314,20 @@ namespace
       if (opt_nondet_states_set)
         matched &= opt_nondet_states.contains(spot::count_nondet_states(aut));
       if (opt_is_deterministic)
-        matched &= is_deterministic(aut);
-      else if (opt_is_unambiguous)
-        matched &= is_unambiguous(aut);
+        {
+          matched &= is_deterministic(aut);
+        }
+      else
+        {
+          if (opt_is_unambiguous)
+            matched &= is_unambiguous(aut);
+          if (opt_is_semi_deterministic)
+            matched &= is_semi_deterministic(aut);
+        }
       if (opt_is_terminal)
         matched &= is_terminal_automaton(aut);
+      else if (opt_is_very_weak)
+        matched &= is_very_weak_automaton(aut);
       else if (opt_is_weak)
         matched &= is_weak_automaton(aut);
       else if (opt_is_inherently_weak)
@@ -1071,13 +1337,13 @@ namespace
       if (opt_is_empty)
         matched &= aut->is_empty();
       if (opt->intersect)
-        matched &= !spot::product(aut, opt->intersect)->is_empty();
+        matched &= aut->intersects(opt->intersect);
       if (opt->included_in)
-        matched &= spot::product(aut, opt->included_in)->is_empty();
+        matched &= !aut->intersects(opt->included_in);
       if (opt->equivalent_pos)
-        matched &= spot::product(aut, opt->equivalent_neg)->is_empty()
-          && spot::product(dtwa_complement(ensure_deterministic(aut)),
-                           opt->equivalent_pos)->is_empty();
+        matched &= !aut->intersects(opt->equivalent_neg)
+          && (!dualize(ensure_deterministic(aut, true))->
+              intersects(opt->equivalent_pos));
 
       if (matched && !opt->acc_words.empty())
         for (auto& word_aut: opt->acc_words)
@@ -1118,14 +1384,15 @@ namespace
         aut = opt->excl_ap.constrain(aut, false);
 
       if (opt_destut)
-        aut = spot::closure(std::move(aut));
+        aut = spot::closure_inplace(std::move(aut));
       if (opt_instut == 1)
         aut = spot::sl(std::move(aut));
       else if (opt_instut == 2)
-        aut = spot::sl2(std::move(aut));
+        aut = spot::sl2_inplace(std::move(aut));
 
       if (!opt_keep_states.empty())
-        aut = mask_keep_states(aut, opt_keep_states, opt_keep_states_initial);
+        aut = mask_keep_accessible_states(aut, opt_keep_states,
+                                          opt_keep_states_initial);
       if (opt_rem_dead)
         aut->purge_dead_states();
       else if (opt_rem_unreach)
@@ -1136,9 +1403,14 @@ namespace
       if (opt->product_or)
         aut = spot::product_or(std::move(aut), opt->product_or);
 
-      if (opt_decompose_strength)
+      if (opt->sum_or)
+        aut = spot::sum(std::move(aut), opt->sum_or);
+      if (opt->sum_and)
+        aut = spot::sum_and(std::move(aut), opt->sum_and);
+
+      if (opt_decompose_scc)
         {
-          aut = decompose_strength(aut, opt_decompose_strength);
+          aut = decompose_scc(aut, opt_decompose_scc);
           if (!aut)
             return 0;
         }
@@ -1151,7 +1423,10 @@ namespace
         }
 
       if (opt_complement)
-        aut = spot::dtwa_complement(ensure_deterministic(aut));
+        aut = spot::dualize(ensure_deterministic(aut));
+
+      if (opt_dualize)
+        aut = spot::dualize(aut);
 
       aut = post.run(aut, nullptr);
 
@@ -1160,11 +1435,17 @@ namespace
       if (opt_gsa)
         aut = spot::to_generalized_streett(aut, opt_gsa == GSA_SHARE_FIN);
 
+      if (opt_streett_like)
+        aut = spot::dnf_to_streett(aut);
+
       if (opt_simplify_exclusive_ap && !opt->excl_ap.empty())
         aut = opt->excl_ap.constrain(aut, true);
       else if (opt_rem_unused_ap) // constrain(aut, true) already does that
         aut->remove_unused_ap();
 
+      if (opt_split_edges)
+        aut = spot::split_edges(aut);
+
       if (randomize_st || randomize_tr)
         spot::randomize(aut, randomize_st, randomize_tr);
 
@@ -1173,6 +1454,9 @@ namespace
       if (opt_highlight_nondet_edges >= 0)
         spot::highlight_nondet_edges(aut, opt_highlight_nondet_edges);
 
+      if (opt_highlight_languages)
+        spot::highlight_languages(aut);
+
       if (!opt->hl_words.empty())
         for (auto& word_aut: opt->hl_words)
           {
@@ -1183,8 +1467,7 @@ namespace
               run->project(aut)->highlight(word_aut.second);
           }
 
-      const double conversion_time = sw.stop();
-
+      timer.stop();
       if (opt->uniq)
         {
           auto tmp =
@@ -1197,8 +1480,8 @@ namespace
 
       ++match_count;
 
-      printer.print(aut, nullptr, filename, -1, conversion_time, haut,
-                    prefix, suffix);
+      printer.print(aut, timer, nullptr, haut->filename.c_str(), -1,
+                    haut, prefix, suffix);
 
       if (opt_max_count >= 0 && match_count >= opt_max_count)
         abort_run = true;
@@ -1249,11 +1532,11 @@ main(int argc, char** argv)
       spot::srand(opt_seed);
 
       spot::postprocessor post(&extra_options);
-      post.set_pref(pref | comp | sbacc);
       post.set_type(type);
+      post.set_pref(pref | comp | sbacc | colored);
       post.set_level(level);
 
-      hoa_processor processor(post);
+      autfilt_processor processor(post, o.dict);
       if (processor.run())
         return 2;
 
@@ -1272,5 +1555,6 @@ main(int argc, char** argv)
   if (automaton_format == Count)
     std::cout << match_count << std::endl;
 
+  check_cout();
   return !match_count;
 }
