@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2014, 2015, 2016  Laboratoire de
+# Copyright (C) 2014-2018  Laboratoire de
 # Recherche et Développement de l'Epita (LRDE).
 #
 # This file is part of Spot, a model checking library.
@@ -122,13 +122,25 @@ class twa:
         return SVG(self._repr_svg_(opt))
 
     def highlight_states(self, states, color):
-        for state in states:
-            self.highlight_state(state, color)
+        """Highlight a list of states.  This can be a list of
+        state numbers, or a list of Booleans."""
+        for idx, val in enumerate(states):
+            if type(val) is bool:
+                if val:
+                    self.highlight_state(idx, color)
+            else:
+                self.highlight_state(val, color)
         return self
 
     def highlight_edges(self, edges, color):
-        for edge in edges:
-            self.highlight_edge(edge, color)
+        """Highlight a list of edges.  This can be a list of
+        edge numbers, or a list of Booleans."""
+        for idx, val in enumerate(edges):
+            if type(val) is bool:
+                if val:
+                    self.highlight_edge(idx, color)
+            else:
+                self.highlight_edge(val, color)
         return self
 
 @_extend(twa)
@@ -166,7 +178,10 @@ class twa:
 class formula:
     def __init__(self, str):
         """Parse the given string to create a formula."""
-        self.this = parse_formula(str)
+        if type(str) == formula:
+            self.this = str
+        else:
+            self.this = parse_formula(str)
 
     def show_ast(self):
         """Display the syntax tree of the formula."""
@@ -282,39 +297,40 @@ class formula:
 
         return s.__format__(spec)
 
-    def traverse(self, func):
-        if func(self):
+    def traverse(self, func, *args):
+        if func(self, *args):
             return
         for f in self:
-            f.traverse(func)
+            f.traverse(func, *args)
 
-    def map(self, func):
+    def map(self, func, *args):
         k = self.kind()
         if k in (op_ff, op_tt, op_eword, op_ap):
             return self
         if k in (op_Not, op_X, op_F, op_G, op_Closure,
                  op_NegClosure, op_NegClosureMarked):
-            return formula.unop(k, func(self[0]))
+            return formula.unop(k, func(self[0], *args))
         if k in (op_Xor, op_Implies, op_Equiv, op_U, op_R, op_W,
                  op_M, op_EConcat, op_EConcatMarked, op_UConcat):
-            return formula.binop(k, func(self[0]), func(self[1]))
+            return formula.binop(k, func(self[0], *args), func(self[1], *args))
         if k in (op_Or, op_OrRat, op_And, op_AndRat, op_AndNLM,
                  op_Concat, op_Fusion):
-            return formula.multop(k, [func(x) for x in self])
+            return formula.multop(k, [func(x, *args) for x in self])
         if k in (op_Star, op_FStar):
-            return formula.bunop(k, func(self[0]), self.min(), self.max())
+            return formula.bunop(k, func(self[0], *args),
+                                 self.min(), self.max())
         raise ValueError("unknown type of formula")
 
 
 def automata(*sources, timeout=None, ignore_abort=True,
-             trust_hoa=True, debug=False):
+             trust_hoa=True, no_sid=False, debug=False):
     """Read automata from a list of sources.
 
     Parameters
     ----------
     *sources : list of str
         These sources can be either commands (end with `|`),
-        textual represantations of automata (contain `\n`),
+        textual representations of automata (contain `\n`),
         or filenames (else).
     timeout : int, optional
         Number of seconds to wait for the result of a command.
@@ -326,6 +342,10 @@ def automata(*sources, timeout=None, ignore_abort=True,
     trust_hoa : bool, optional
         If True (the default), supported HOA properies that
         cannot be easily verified are trusted.
+    no_sid : bool, optional
+        When an automaton is obtained from a subprocess, this
+        subprocess is started from a shell with its own session
+        group (the default) unless no_sid is set to True.
     debug : bool, optional
         Whether to run the parser in debug mode.
 
@@ -386,11 +406,15 @@ def automata(*sources, timeout=None, ignore_abort=True,
             p = None
             proc = None
             if filename[-1] == '|':
+                setsid_maybe = None
+                if not no_sid:
+                    setsid_maybe = os.setsid
                 # universal_newlines for str output instead of bytes
                 # when the pipe is read from Python (which happens
                 # when timeout is set).
                 proc = subprocess.Popen(filename[:-1], shell=True,
-                                        preexec_fn=os.setsid,
+                                        preexec_fn=
+                                        None if no_sid else os.setsid,
                                         universal_newlines=True,
                                         stdout=subprocess.PIPE)
                 if timeout is None:
@@ -466,6 +490,7 @@ def _postproc_translate_options(obj, default_type, *args):
     comp_ = 0
     unam_ = 0
     sbac_ = 0
+    colo_ = 0
 
     def type_set(val):
         nonlocal type_
@@ -478,6 +503,29 @@ def _postproc_translate_options(obj, default_type, *args):
             type_ = postprocessor.TGBA
         elif val == 'ba':
             type_ = postprocessor.BA
+        elif val == 'cobuchi' or val == 'nca':
+            type_ = postprocessor.CoBuchi
+        elif val == 'dca':
+            type_ = postprocessor.CoBuchi
+            pref_ = postprocessor.Deterministic
+        elif val == 'parity min odd':
+            type_ = postprocessor.ParityMinOdd
+        elif val == 'parity min even':
+            type_ = postprocessor.ParityMinEven
+        elif val == 'parity max odd':
+            type_ = postprocessor.ParityMaxOdd
+        elif val == 'parity max even':
+            type_ = postprocessor.ParityMaxEven
+        elif val == 'parity min':
+            type_ = postprocessor.ParityMin
+        elif val == 'parity max':
+            type_ = postprocessor.ParityMax
+        elif val == 'parity odd':
+            type_ = postprocessor.ParityOdd
+        elif val == 'parity even':
+            type_ = postprocessor.ParityEven
+        elif val == 'parity':
+            type_ = postprocessor.Parity
         else:
             assert(val == 'monitor')
             type_ = postprocessor.Monitor
@@ -509,8 +557,10 @@ def _postproc_translate_options(obj, default_type, *args):
             optm_ = postprocessor.Low
 
     def misc_set(val):
-        nonlocal comp_, unam_, sbac_
-        if val == 'complete':
+        nonlocal comp_, unam_, sbac_, colo_
+        if val == 'colored':
+            colo_ = postprocessor.Colored
+        elif val == 'complete':
             comp_ = postprocessor.Complete
         elif val == 'sbacc' or val == 'state-based-acceptance':
             sbac_ = postprocessor.SBAcc
@@ -519,20 +569,33 @@ def _postproc_translate_options(obj, default_type, *args):
             unam_ = postprocessor.Unambiguous
 
     options = {
-        'tgba': type_set,
-        'ba': type_set,
-        'monitor': type_set,
-        'generic': type_set,
-        'small': pref_set,
-        'deterministic': pref_set,
         'any': pref_set,
-        'high': optm_set,
-        'medium': optm_set,
-        'low': optm_set,
+        'ba': type_set,
+        'cobuchi': type_set,
+        'colored': misc_set,
         'complete': misc_set,
-        'unambiguous': misc_set,
-        'statebasedacceptance': misc_set,
+        'dca': type_set,
+        'deterministic': pref_set,
+        'generic': type_set,
+        'high': optm_set,
+        'low': optm_set,
+        'medium': optm_set,
+        'monitor': type_set,
+        'nca': type_set,
+        'parity even': type_set,
+        'parity max even': type_set,
+        'parity max odd': type_set,
+        'parity max': type_set,
+        'parity min even': type_set,
+        'parity min odd': type_set,
+        'parity min': type_set,
+        'parity odd': type_set,
+        'parity': type_set,
         'sbacc': misc_set,
+        'small': pref_set,
+        'statebasedacceptance': misc_set,
+        'tgba': type_set,
+        'unambiguous': misc_set,
     }
 
     for arg in args:
@@ -566,7 +629,7 @@ def _postproc_translate_options(obj, default_type, *args):
         optm_ = postprocessor.High
 
     obj.set_type(type_)
-    obj.set_pref(pref_ | comp_ | unam_ | sbac_)
+    obj.set_pref(pref_ | comp_ | unam_ | sbac_ | colo_)
     obj.set_level(optm_)
 
 
@@ -577,14 +640,17 @@ def translate(formula, *args, dict=_bdd_dict):
     may not be satisfied.
 
     The optional arguments should be strings among the following:
-    - at most one in 'TGBA', 'BA', or 'Monitor', 'generic'
-      (type of automaton to build)
+    - at most one in 'TGBA', 'BA', or 'Monitor', 'generic',
+      'parity', 'parity min odd', 'parity min even',
+      'parity max odd', 'parity max even' (type of automaton to
+      build), 'coBuchi'
     - at most one in 'Small', 'Deterministic', 'Any'
       (preferred characteristics of the produced automaton)
     - at most one in 'Low', 'Medium', 'High'
       (optimization level)
-    - any combination of 'Complete', 'Unambiguous', and
-      'StateBasedAcceptance' (or 'SBAcc' for short)
+    - any combination of 'Complete', 'Unambiguous',
+      'StateBasedAcceptance' (or 'SBAcc' for short), and
+      'Colored' (only for parity acceptance)
 
     The default corresponds to 'tgba', 'small' and 'high'.
     """
@@ -607,14 +673,17 @@ def postprocess(automaton, *args, formula=None):
     not already 'Deterministic'.
 
     The optional arguments should be strings among the following:
-    - at most one in 'Generic', 'TGBA', 'BA', or 'Monitor'
-      (type of automaton to build)
+    - at most one in 'Generic', 'TGBA', 'BA', or 'Monitor',
+      'parity', 'parity min odd', 'parity min even',
+      'parity max odd', 'parity max even' (type of automaton to
+      build), 'coBuchi'
     - at most one in 'Small', 'Deterministic', 'Any'
       (preferred characteristics of the produced automaton)
     - at most one in 'Low', 'Medium', 'High'
       (optimization level)
-    - any combination of 'Complete' and 'StateBasedAcceptance'
-      (or 'SBAcc' for short)
+    - any combination of 'Complete', 'StateBasedAcceptance'
+      (or 'SBAcc' for short), and 'Colored (only for parity
+      acceptance)
 
     The default corresponds to 'generic', 'small' and 'high'.
 
@@ -842,7 +911,8 @@ for fun in dir(formula):
         _addfilter(fun)
 
 for fun in ['remove_x', 'relabel', 'relabel_bse',
-            'simplify', 'unabbreviate']:
+            'simplify', 'unabbreviate', 'negative_normal_form',
+            'mp_class', 'nesting_depth']:
     _addmap(fun)
 
 
@@ -850,7 +920,8 @@ for fun in ['remove_x', 'relabel', 'relabel_bse',
 # Better interface to the corresponding C++ function.
 def sat_minimize(aut, acc=None, colored=False,
                  state_based=False, states=0,
-                 max_states=0, dichotomy=False):
+                 max_states=0, sat_naive=False, sat_langmap=False,
+                 sat_incr=0, sat_incr_steps=0):
     args=''
     if acc is not None:
         if type(acc) is not str:
@@ -866,15 +937,179 @@ def sat_minimize(aut, acc=None, colored=False,
         if type(max_states) is not int or max_states < 0:
             raise ValueError("argument 'states' should be a positive integer")
         args += ',max-states=' + str(max_states)
-    if dichotomy:
-        args += ',dichotomy';
+    if sat_naive:
+        args += ',sat-naive'
+    if sat_langmap:
+        args += ',sat-langmap'
+    if sat_incr:
+        args += ',sat-incr=' + str(sat_incr)
+        args += ',sat-incr-steps=' + str(sat_incr_steps)
+
     from spot.impl import sat_minimize as sm
     return sm(aut, args, state_based)
 
+
 def parse_word(word, dic=_bdd_dict):
     from spot.impl import parse_word as pw
     return pw(word, dic)
 
+def bdd_to_formula(b, dic=_bdd_dict):
+    from spot.impl import bdd_to_formula as bf
+    return bf(b, dic)
+
 def language_containment_checker(dic=_bdd_dict):
     from spot.impl import language_containment_checker as c
     return c(dic)
+
+
+def mp_hierarchy_svg(cl=None):
+    """
+    Return an some string containing an SVG picture of the Manna &
+    Pnueli hierarchy, highlighting class `cl` if given.
+
+    If not None, `cl` should be one of 'TPROGSB'.  For convenience,
+    if `cl` is an instance of `spot.formula`, it is replaced by
+    `mp_class(cl)`.
+
+    """
+    if type(cl)==formula:
+        cl = mp_class(cl)
+    ch = None
+    coords = {
+        'T': '110,35',
+        'R': '40,80',
+        'P': '175,80',
+        'O': '110,140',
+        'S': '40,160',
+        'G': '175,160',
+        'B': '110,198',
+    }
+    if cl in coords:
+        highlight='''<g transform="translate({})">
+    <line x1="-10" y1="-10" x2="10" y2="10" stroke="red" stroke-width="5" />
+    <line x1="-10" y1="10" x2="10" y2="-10" stroke="red" stroke-width="5" />
+    </g>'''.format(coords[cl])
+    else:
+        highlight=''
+    return '''
+<svg height="210" width="220" xmlns="http://www.w3.org/2000/svg" version="1.1">
+<polygon points="20,0 200,120 200,210 20,210" fill="cyan" opacity=".2" />
+<polygon points="20,120 155,210 20,210" fill="cyan" opacity=".2" />
+<polygon points="200,0 20,120 20,210 200,210" fill="magenta" opacity=".15" />
+<polygon points="200,120 65,210 200,210" fill="magenta" opacity=".15" />
+''' + highlight + '''
+<g text-anchor="middle" font-size="14">
+<text x="110" y="20">Reactivity</text>
+<text x="60" y="65">Recurrence</text>
+<text x="160" y="65">Persistence</text>
+<text x="110" y="125">Obligation</text>
+<text x="60" y="185">Safety</text>
+<text x="160" y="185">Guarantee</text>
+</g>
+<g font-size="14">
+<text text-anchor="begin" transform="rotate(-90,18,210)" x="18" y="210" fill="gray">Monitor</text>
+<text text-anchor="end" transform="rotate(-90,18,0)" x="18" y="0" fill="gray">Deterministic Büchi</text>
+<text text-anchor="begin" transform="rotate(-90,214,210)" x="214" y="210" fill="gray">Terminal Büchi</text>
+<text text-anchor="end" transform="rotate(-90,214,0)" x="214" y="0" fill="gray">Weak Büchi</text>
+</g>
+</svg>'''
+
+
+def show_mp_hierarchy(cl):
+    """
+    Return a picture of the Manna & Pnueli hierarchy as an SVG object
+    in the IPython/Jupyter.
+    """
+    from IPython.display import SVG
+    return SVG(mp_hierarchy_svg(cl))
+
+formula.show_mp_hierarchy = show_mp_hierarchy
+
+@_extend(twa_word)
+class twa_word:
+    def as_svg(self):
+        """
+        Build an SVG picture representing the word as a collection of
+        signals for each atomic proposition.
+        """
+        # Get the list of atomic proposition used
+        sup = buddy.bddtrue
+        for cond in list(self.prefix) + list(self.cycle):
+            sup = sup & buddy.bdd_support(cond)
+        ap = []
+        while sup != buddy.bddtrue:
+            a = buddy.bdd_var(sup)
+            ap.append(a)
+            sup = buddy.bdd_high(sup)
+
+        # Prepare canvas
+        psize = len(self.prefix)
+        csize = len(self.cycle)
+        d = {
+            'endprefix': 50 * psize,
+            'endcycle': 50 * (psize + csize),
+            'w': 50 * (psize + csize * 2),
+            'height': 50 * len(ap),
+            'height2': 50 * len(ap) + 10,
+            'h3': 50 * len(ap) + 12,
+            'bgcolor': '#f4f4f4',
+            'bgl': 'stroke="white" stroke-width="4"',
+            'bgt': 'stroke="white" stroke-width="1"',
+            'txt': 'text-anchor="start" font-size="20"',
+            'red': 'stroke="#ff0000" stroke-width="2"',
+            'sml': 'text-anchor="start" font-size="10"'
+            }
+        txt = '''
+<svg height="{h3}" width="{w}" xmlns="http://www.w3.org/2000/svg" version="1.1">
+<rect x="0" y="0" width="{w}" height="{height}" fill="{bgcolor}"/>
+<line x1="{endprefix}" y1="0" x2="{endprefix}" y2="{height}"
+      stroke="white" stroke-width="4"/>
+<line x1="{endcycle}" y1="0" x2="{endcycle}" y2="{height}"
+      stroke="white" stroke-width="4"/>
+'''.format(**d)
+
+        # Iterate over all used atomic propositions, and fill each line
+        l = list(self.prefix) + list(self.cycle) + list(self.cycle)
+        bd = self.get_dict()
+        for ypos, a in enumerate(ap):
+            pa = buddy.bdd_ithvar(a)
+            na = buddy.bdd_nithvar(a)
+            name = bdd_format_formula(bd, pa)
+            # Whether the last state was down (-1), up (1), or unknown (0)
+            last = 0
+            txt += ('<line x1="0" y1="{y}" x2="{w}" y2="{y}" {bgl}/>'
+                    .format(y=ypos*50, **d))
+            txt += ('<text x="{x}" y="{y}" {txt}>{name}</text>'
+                    .format(x=3, y=ypos*50+30, name=name, **d))
+            for xpos, step in enumerate(l):
+                if buddy.bdd_implies(step, pa):
+                    cur = 1
+                elif buddy.bdd_implies(step, na):
+                    cur = -1
+                else:
+                    cur = 0
+                txt += ('<line x1="{x}" y1="{y1}" x2="{x}" y2="{y2}" {bgt}/>'
+                        .format(x=(xpos+1)*50, y1=ypos*50, y2=ypos*50+50, **d))
+                if cur != 0:
+                    if last == -cur:
+                        txt += \
+                        ('<line x1="{x}" y1="{y1}" x2="{x}" y2="{y2}" {red}/>'
+                             .format(x=xpos*50, y1=ypos*50+5,
+                                     y2=ypos*50+45, **d))
+                    txt += \
+                    ('<line x1="{x1}" y1="{y}" x2="{x2}" y2="{y}" {red}/>'
+                         .format(x1=xpos*50, x2=(xpos+1)*50,
+                                 y=ypos*50+25-20*cur, **d))
+                last = cur
+        if psize > 0:
+            txt += '<text x="0" y="{height2}" {sml}>prefix</text>'.format(**d)
+        txt += '''<text x="{endprefix}" y="{height2}" {sml}>cycle</text>
+<text x="{endcycle}" y="{height2}" {sml}>cycle</text>'''.format(**d)
+        return txt + '</svg>'
+
+    def show(self):
+        """
+        Display the word as an SVG picture of signals.
+        """
+        from IPython.display import SVG
+        return SVG(self.as_svg())
