@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2012, 2014, 2015, 2016 Laboratoire de Recherche et
-// Développement de l'Epita (LRDE)
+// Copyright (C) 2011, 2012, 2014-2018 Laboratoire de
+// Recherche et Développement de l'Epita (LRDE)
 //
 // This file is part of Spot, a model checking library.
 //
@@ -121,7 +121,6 @@ namespace spot
         if (this == other)
           return 0;
         const spins_state* o = down_cast<const spins_state*>(other);
-        assert(o);
         if (hash_value < o->hash_value)
           return -1;
         if (hash_value > o->hash_value)
@@ -140,7 +139,7 @@ namespace spot
       size_t hash_value: 32;
       int size: 16;
       mutable unsigned count: 16;
-      int vars[0];
+      int vars[1];
     };
 
     struct spins_compressed_state final: public state
@@ -167,7 +166,8 @@ namespace spot
       {
         if (--count)
           return;
-        pool->deallocate(this, sizeof(*this) + size * sizeof(*vars));
+        pool->deallocate(this, sizeof(*this) - sizeof(vars)
+                         + size * sizeof(*vars));
       }
 
       size_t hash() const override
@@ -181,7 +181,6 @@ namespace spot
           return 0;
         const spins_compressed_state* o =
           down_cast<const spins_compressed_state*>(other);
-        assert(o);
         if (hash_value < o->hash_value)
           return -1;
         if (hash_value > o->hash_value)
@@ -206,7 +205,7 @@ namespace spot
       size_t hash_value: 32;
       int size: 16;
       mutable unsigned count: 16;
-      int vars[0];
+      int vars[1];
     };
 
     ////////////////////////////////////////////////////////////////////////
@@ -236,7 +235,7 @@ namespace spot
         new(p->allocate()) spins_state(ctx->state_size, p);
       memcpy(out->vars, dst, ctx->state_size * sizeof(int));
       out->compute_hash();
-      ctx->transitions.push_back(out);
+      ctx->transitions.emplace_back(out);
     }
 
     void transition_callback_compress(void* arg, transition_info_t*, int *dst)
@@ -248,11 +247,12 @@ namespace spot
       ctx->compress(dst, ctx->state_size, ctx->compressed, csize);
 
       void* mem = p->allocate(sizeof(spins_compressed_state)
+                              - sizeof(spins_compressed_state::vars)
                               + sizeof(int) * csize);
       spins_compressed_state* out = new(mem) spins_compressed_state(csize, p);
       memcpy(out->vars, ctx->compressed, csize * sizeof(int));
       out->compute_hash();
-      ctx->transitions.push_back(out);
+      ctx->transitions.emplace_back(out);
     }
 
     ////////////////////////////////////////////////////////////////////////
@@ -462,7 +462,7 @@ namespace spot
               // Record that X.Y must be equal to Z.
               int v = dict->register_proposition(*ap, d.get());
               one_prop p = { ni->second.num, OP_EQ, ei->second, v };
-              out.push_back(p);
+              out.emplace_back(p);
               free(name);
               continue;
             }
@@ -473,7 +473,7 @@ namespace spot
             {
               int v = dict->register_proposition(*ap, d);
               one_prop p = { var_num, OP_NE, 0, v };
-              out.push_back(p);
+              out.emplace_back(p);
               free(name);
               continue;
             }
@@ -591,7 +591,7 @@ namespace spot
 
           int v = dict->register_proposition(*ap, d);
           one_prop p = { var_num, op, val, v };
-          out.push_back(p);
+          out.emplace_back(p);
         }
 
       if (errors)
@@ -620,8 +620,11 @@ namespace spot
                       : int_array_array_decompress2),
           uncompressed_(compress ? new int[state_size_ + 30] : nullptr),
           compressed_(compress ? new int[state_size_ * 2] : nullptr),
-          statepool_(compress ? sizeof(spins_compressed_state) :
-                     (sizeof(spins_state) + state_size_ * sizeof(int))),
+          statepool_(compress ?
+                     (sizeof(spins_compressed_state)
+                      - sizeof(spins_compressed_state::vars)) :
+                     (sizeof(spins_state) - sizeof(spins_state::vars)
+                      + (state_size_ * sizeof(int)))),
           state_condition_last_state_(nullptr),
           state_condition_last_cc_(nullptr)
       {
@@ -703,6 +706,7 @@ namespace spot
             multiple_size_pool* p =
               const_cast<multiple_size_pool*>(&compstatepool_);
             void* mem = p->allocate(sizeof(spins_compressed_state)
+                                    - sizeof(spins_compressed_state::vars)
                                     + sizeof(int) * csize);
             spins_compressed_state* res = new(mem)
               spins_compressed_state(csize, p);
@@ -809,7 +813,7 @@ namespace spot
 
             // Add a self-loop to dead-states if we care about these.
             if (res != bddfalse)
-              cc->transitions.push_back(st->clone());
+              cc->transitions.emplace_back(st->clone());
           }
 
         state_condition_last_cc_ = cc;
@@ -827,7 +831,6 @@ namespace spot
           {
             const spins_compressed_state* s =
               down_cast<const spins_compressed_state*>(st);
-            assert(s);
 
             decompress_(s->vars, s->size, uncompressed_, state_size_);
             vars = uncompressed_;
@@ -835,7 +838,6 @@ namespace spot
         else
           {
             const spins_state* s = down_cast<const spins_state*>(st);
-            assert(s);
             vars = s->vars;
           }
         return vars;
@@ -861,7 +863,7 @@ namespace spot
 
             // Add a self-loop to dead-states if we care about these.
             if (t == 0 && scond != bddfalse)
-              cc->transitions.push_back(st->clone());
+              cc->transitions.emplace_back(st->clone());
           }
 
         if (iter_cache_)
@@ -943,12 +945,17 @@ namespace spot
     // Call spins to compile "foo.prom" as "foo.prom.spins" if the latter
     // does not exist already or is older.
     static void
-    compile_model(std::string& filename, std::string& ext)
+    compile_model(std::string& filename, const std::string& ext)
     {
       std::string command;
       std::string compiled_ext;
 
-      if (ext == ".prom" || ext == ".pm" || ext == ".pml")
+      if (ext == ".gal")
+        {
+          command = "gal2c " + filename;
+          compiled_ext = "2C";
+        }
+      else if (ext == ".prom" || ext == ".pm" || ext == ".pml")
         {
           command = "spins " + filename;
           compiled_ext = ".spins";
@@ -962,14 +969,14 @@ namespace spot
         {
           throw std::runtime_error(std::string("Unknown extension '")
                                    + ext + "'.  Use '.prom', '.pm', '.pml', "
-                                   "'.dve', '.dve2C', or '.prom.spins'.");
+                                   "'.dve', '.dve2C', '.gal', '.gal2C' or "
+                                   "'.prom.spins'.");
         }
 
       struct stat s;
       if (stat(filename.c_str(), &s) != 0)
         throw std::runtime_error(std::string("Cannot open ") + filename);
 
-      std::string old = filename;
       filename += compiled_ext;
 
       // Remove any directory, because the new file will
@@ -981,7 +988,7 @@ namespace spot
       struct stat d;
       if (stat(filename.c_str(), &d) == 0)
         if (s.st_mtime < d.st_mtime)
-          // The .spins or .dve2C is up-to-date, no need to recompile it.
+          // The .spins or .dve2C or .gal2C is up-to-date, no need to compile.
           return;
 
       int res = system(command.c_str());
@@ -1003,8 +1010,11 @@ namespace spot
       file = "./" + file_arg;
 
     std::string ext = file.substr(file.find_last_of("."));
-    if (ext != ".spins" && ext != ".dve2C")
-      compile_model(file, ext);
+    if (ext != ".spins" && ext != ".dve2C" && ext != ".gal2C")
+      {
+        compile_model(file, ext);
+        ext = file.substr(file.find_last_of("."));
+      }
 
     if (lt_dlinit())
       throw std::runtime_error("Failed to initialize libltldl.");
@@ -1012,73 +1022,59 @@ namespace spot
     lt_dlhandle h = lt_dlopen(file.c_str());
     if (!h)
       {
+        std::string lt_error = lt_dlerror();
         lt_dlexit();
         throw std::runtime_error(std::string("Failed to load '")
-                                 + file + "'.");
+                                 + file + "'.\n" + lt_error);
       }
 
     auto d = std::make_shared<spins_interface>();
+    assert(d); // Superfluous, but Debian's GCC 7 snapshot 20161207-1 warns
+               // about potential null pointer dereference on the next line.
     d->handle = h;
 
-    // SpinS interface.
-    if ((d->get_initial_state = (void (*)(void*))
-        lt_dlsym(h, "spins_get_initial_state")))
+
+    auto sym = [&](auto* dst, const char* name)
       {
+        // Work around -Wpendantic complaining that pointer-to-objects
+        // should not be converted to pointer-to-functions (we have to
+        // assume they can for POSIX).
+        *reinterpret_cast<void**>(dst) = lt_dlsym(h, name);
+        if (dst == nullptr)
+          throw std::runtime_error(std::string("Failed to resolve symbol '")
+                                   + name + "' in '" + file + "'.");
+      };
+
+    // SpinS interface.
+    if (ext == ".spins")
+      {
+        sym(&d->get_initial_state, "spins_get_initial_state");
         d->have_property = nullptr;
-        d->get_successors = (int (*)(void*, int*, TransitionCB, void*))
-        lt_dlsym(h, "spins_get_successor_all");
-        d->get_state_size = (int (*)())
-        lt_dlsym(h, "spins_get_state_size");
-        d->get_state_variable_name = (const char* (*)(int))
-        lt_dlsym(h, "spins_get_state_variable_name");
-        d->get_state_variable_type = (int (*)(int))
-        lt_dlsym(h, "spins_get_state_variable_type");
-        d->get_type_count = (int (*)())
-        lt_dlsym(h, "spins_get_type_count");
-        d->get_type_name = (const char* (*)(int))
-        lt_dlsym(h, "spins_get_type_name");
-        d->get_type_value_count = (int (*)(int))
-        lt_dlsym(h, "spins_get_type_value_count");
-        d->get_type_value_name = (const char* (*)(int, int))
-        lt_dlsym(h, "spins_get_type_value_name");
+        sym(&d->get_successors, "spins_get_successor_all");
+        sym(&d->get_state_size, "spins_get_state_size");
+        sym(&d->get_state_variable_name, "spins_get_state_variable_name");
+        sym(&d->get_state_variable_type, "spins_get_state_variable_type");
+        sym(&d->get_type_count, "spins_get_type_count");
+        sym(&d->get_type_name, "spins_get_type_name");
+        sym(&d->get_type_value_count, "spins_get_type_value_count");
+        sym(&d->get_type_value_name, "spins_get_type_value_name");
       }
-    // dve2 interface.
+    // dve2 and gal2C interfaces.
     else
       {
-        d->get_initial_state = (void (*)(void*))
-        lt_dlsym(h, "get_initial_state");
-        d->have_property = (int (*)())
-        lt_dlsym(h, "have_property");
-        d->get_successors = (int (*)(void*, int*, TransitionCB, void*))
-        lt_dlsym(h, "get_successors");
-        d->get_state_size = (int (*)())
-        lt_dlsym(h, "get_state_variable_count");
-        d->get_state_variable_name = (const char* (*)(int))
-        lt_dlsym(h, "get_state_variable_name");
-        d->get_state_variable_type = (int (*)(int))
-        lt_dlsym(h, "get_state_variable_type");
-        d->get_type_count = (int (*)())
-        lt_dlsym(h, "get_state_variable_type_count");
-        d->get_type_name = (const char* (*)(int))
-        lt_dlsym(h, "get_state_variable_type_name");
-        d->get_type_value_count = (int (*)(int))
-        lt_dlsym(h, "get_state_variable_type_value_count");
-        d->get_type_value_name = (const char* (*)(int, int))
-        lt_dlsym(h, "get_state_variable_type_value");
+        sym(&d->get_initial_state, "get_initial_state");
+        *reinterpret_cast<void**>(&d->have_property) =
+          lt_dlsym(h, "have_property");
+        sym(&d->get_successors, "get_successors");
+        sym(&d->get_state_size, "get_state_variable_count");
+        sym(&d->get_state_variable_name, "get_state_variable_name");
+        sym(&d->get_state_variable_type, "get_state_variable_type");
+        sym(&d->get_type_count, "get_state_variable_type_count");
+        sym(&d->get_type_name, "get_state_variable_type_name");
+        sym(&d->get_type_value_count, "get_state_variable_type_value_count");
+        sym(&d->get_type_value_name, "get_state_variable_type_value");
       }
 
-    if (!(d->get_initial_state
-          && d->get_successors
-          && d->get_state_size
-          && d->get_state_variable_name
-          && d->get_state_variable_type
-          && d->get_type_count
-          && d->get_type_name
-          && d->get_type_value_count
-          && d->get_type_value_name))
-      throw std::runtime_error(std::string("Failed resolve some symbol"
-                                           "while loading '") + file + "'.");
-
     if (d->have_property && d->have_property())
       throw std::runtime_error("Models with embedded properties "
                                "are not supported.");
@@ -1097,7 +1093,7 @@ namespace spot
       {
         convert_aps(to_observe, iface, dict, dead, *ps);
       }
-    catch (std::runtime_error)
+    catch (const std::runtime_error&)
       {
         delete ps;
         dict->unregister_all_my_variables(iface.get());
@@ -1109,6 +1105,8 @@ namespace spot
     // twa::ap() works.
     for (auto ap: *to_observe)
       res->register_ap(ap);
+    if (dead.is(op::ap))
+      res->register_ap(dead);
     return res;
   }
 
