@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2009, 2011, 2012, 2013, 2014, 2015, 2016 Laboratoire de
-// Recherche et Développement de l'Epita (LRDE).
+// Copyright (C) 2009, 2011-2017 Laboratoire de Recherche et
+// Développement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005 Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
 // et Marie Curie.
@@ -24,6 +24,7 @@
 #include <memory>
 #include <spot/twaalgos/emptiness.hh>
 #include <spot/twaalgos/bfssteps.hh>
+#include <spot/twaalgos/couvreurnew.hh>
 #include <spot/twaalgos/gtec/gtec.hh>
 #include <spot/twaalgos/gv04.hh>
 #include <spot/twaalgos/magic.hh>
@@ -129,6 +130,8 @@ namespace spot
     ec_algo ec_algos[] =
       {
         { "Cou99",     couvreur99,                    0, -1U },
+        { "Cou99new",  get_couvreur99_new,            0, -1U },
+        { "Cou99abs",  get_couvreur99_new_abstract,   0, -1U },
         { "CVWY90",    magic_search,                  0,   1 },
         { "GV04",      explicit_gv04_check,           0,   1 },
         { "SE05",      se05,                          0,   1 },
@@ -235,12 +238,12 @@ namespace spot
     for (step s : run.prefix)
       {
         s.s = s.s->clone();
-        prefix.push_back(s);
+        prefix.emplace_back(s);
       }
     for (step s : run.cycle)
       {
         s.s = s.s->clone();
-        cycle.push_back(s);
+        cycle.emplace_back(s);
       }
   }
 
@@ -374,7 +377,7 @@ namespace spot
         seen_acc |= seg->acc;
 
         twa_run::step st = { seg->s->clone(), seg->label, seg->acc };
-        res->cycle.push_back(st);
+        res->cycle.emplace_back(st);
 
         ++seg;
       }
@@ -443,11 +446,11 @@ namespace spot
     auto res = std::make_shared<twa_run>(other);
     if (auto ps = aut->get_named_prop<const product_states>("product-states"))
       {
-        auto a = std::dynamic_pointer_cast<const twa_graph>(aut);
+        auto a = down_cast<const_twa_graph_ptr>(aut);
         if (!a)
           throw std::runtime_error("twa_run::project() confused: "
                                    "product-states found in a non-twa_graph");
-        auto oth = std::dynamic_pointer_cast<const twa_graph>(other);
+        auto oth = down_cast<const_twa_graph_ptr>(other);
         if (!oth)
           throw std::runtime_error("twa_run::project() confused: "
                                    "other ought to be a twa_graph");
@@ -706,8 +709,7 @@ namespace spot
   /// Note that this works only if the automaton is a twa_graph_ptr.
   void twa_run::highlight(unsigned color)
   {
-    auto a = std::dynamic_pointer_cast<twa_graph>
-      (std::const_pointer_cast<twa>(aut));
+    auto a = down_cast<twa_graph_ptr>(std::const_pointer_cast<twa>(aut));
     if (!a)
       throw std::runtime_error("highlight() only work for twa_graph");
 
@@ -750,13 +752,20 @@ namespace spot
   }
 
   twa_graph_ptr
-  twa_run::as_twa() const
+  twa_run::as_twa(bool preserve_names) const
   {
     auto d = aut->get_dict();
     auto res = make_twa_graph(d);
     res->copy_ap_of(aut);
     res->copy_acceptance_of(aut);
 
+    std::vector<std::string>* names= nullptr;
+    if (preserve_names)
+      {
+        names = new std::vector<std::string>;
+        res->set_named_prop("state-names", names);
+      }
+
     const state* s = aut->get_init_state();
     unsigned src;
     unsigned dst;
@@ -775,6 +784,8 @@ namespace spot
     assert(s->compare(i->s) == 0);
     src = res->new_state();
     seen.emplace(i->s, src);
+    if (names)
+      names->push_back(aut->format_state(s));
 
     for (; i != l->end();)
       {
@@ -824,7 +835,15 @@ namespace spot
 
         auto p = seen.emplace(next, 0);
         if (p.second)
-          p.first->second = res->new_state();
+          {
+            unsigned ns = res->new_state();
+            p.first->second = ns;
+            if (names)
+              {
+                assert(ns = names->size());
+                names->push_back(aut->format_state(next));
+              }
+          }
         dst = p.first->second;
 
         res->new_edge(src, dst, label, acc);
