@@ -19,7 +19,7 @@ State: 2
 --END--
 """)
 aut.prop_inherently_weak(True)
-aut = spot.dtwa_complement(aut)
+aut = spot.dualize(aut)
 aut1 = spot.scc_filter_states(aut)
 assert(aut1.to_str('hoa') == """HOA: v1
 States: 2
@@ -43,7 +43,7 @@ assert(aut1.get_name() == "test me")
 # The method is the same as the function
 
 a = spot.translate('true', 'low', 'any')
-assert(a.prop_deterministic().is_maybe())
+assert(a.prop_universal().is_maybe())
 assert(a.prop_unambiguous().is_maybe())
 assert(a.is_deterministic() == True)
 assert(a.is_unambiguous() == True)
